@@ -15,8 +15,13 @@ from .core_cython import _validate_order
 __all__ = [
     'nside_to_pixel_area',
     'nside_to_pixel_resolution',
+    'pixel_resolution_to_nside',
     'nside_to_npix',
     'npix_to_nside',
+    'level_to_nside',
+    'nside_to_level',
+    'level_ipix_to_uniq',
+    'uniq_to_level_ipix',
     'lonlat_to_healpix',
     'healpix_to_lonlat',
     'bilinear_interpolation_weights',
@@ -51,7 +56,7 @@ def _validate_offset(label, offset):
 
 
 def _validate_level(level):
-    if level < 0:
+    if np.any(level < 0):
         raise ValueError('level must be positive')
 
 
@@ -61,10 +66,16 @@ def _validate_nside(nside):
         raise ValueError('nside must be a power of two')
 
 
+def _validate_npix(level, ipix):
+    if not np.all(ipix < (3 << 2*(level + 1))):
+        raise ValueError('ipix for a specific level must be inferior to npix')
+
+
 def level_to_nside(level):
     """
-    Find the pixel dimensions of the top-level HEALPix tiles given the
-    resolution level (this is given by 2**level).
+    Find the pixel dimensions of the top-level HEALPix tiles.
+
+    This is given by ``nside = 2**level``.
 
     Parameters
     ----------
@@ -76,10 +87,94 @@ def level_to_nside(level):
     nside : int
         The number of pixels on the side of one of the 12 'top-level' HEALPix tiles.
     """
+    level = np.asarray(level, dtype=np.int64)
+
     _validate_level(level)
     return 2 ** level
 
 
+def nside_to_level(nside):
+    """
+    Find the HEALPix level for a given nside.
+
+    This is given by ``level = log2(nside)``.
+
+    This function is the inverse of `level_to_nside`.
+
+    Parameters
+    ----------
+    nside : int
+        The number of pixels on the side of one of the 12 'top-level' HEALPix tiles.
+        Must be a power of two.
+
+    Returns
+    -------
+    level : int
+        The level of the HEALPix cells
+    """
+    nside = np.asarray(nside, dtype=np.int64)
+
+    _validate_nside(nside)
+    return np.log2(nside).astype(np.int64)
+
+
+def uniq_to_level_ipix(uniq):
+    """
+    Convert a HEALPix cell uniq number to its (level, ipix) equivalent.
+
+    A uniq number is a 64 bits integer equaling to : ipix + 4*(4**level). Please read
+    this `paper <http://ivoa.net/documents/MOC/20140602/REC-MOC-1.0-20140602.pdf>`_
+    for more details about uniq numbers.
+
+    Parameters
+    ----------
+    uniq : int
+        The uniq number of a HEALPix cell.
+
+    Returns
+    -------
+    level, ipix: int, int
+        The level and index of the HEALPix cell computed from ``uniq``.
+    """
+    uniq = np.asarray(uniq, dtype=np.int64)
+
+    level = (np.log2(uniq//4)) // 2
+    level = level.astype(np.int64)
+    _validate_level(level)
+
+    ipix = uniq - (1 << 2*(level + 1))
+    _validate_npix(level, ipix)
+
+    return level, ipix
+
+
+def level_ipix_to_uniq(level, ipix):
+    """
+    Convert a level and HEALPix index into a uniq number representing the cell.
+
+    This function is the inverse of `uniq_to_level_ipix`.
+
+    Parameters
+    ----------
+    level : int
+        The level of the HEALPix cell
+    ipix : int
+        The index of the HEALPix cell
+
+    Returns
+    -------
+    uniq : int
+        The uniq number representing the HEALPix cell.
+    """
+    level = np.asarray(level, dtype=np.int64)
+    ipix = np.asarray(ipix, dtype=np.int64)
+
+    _validate_level(level)
+    _validate_npix(level, ipix)
+
+    return ipix + (1 << 2*(level + 1))
+
+
 def nside_to_pixel_area(nside):
     """
     Find the area of HEALPix pixels given the pixel dimensions of one of
@@ -116,12 +211,78 @@ def nside_to_pixel_resolution(nside):
     -------
     resolution : :class:`~astropy.units.Quantity`
         The resolution of the HEALPix pixels
+
+    See also
+    --------
+    pixel_resolution_to_nside
     """
     nside = np.asanyarray(nside, dtype=np.int64)
     _validate_nside(nside)
     return (nside_to_pixel_area(nside) ** 0.5).to(u.arcmin)
 
 
+def pixel_resolution_to_nside(resolution, round='nearest'):
+    """Find closest HEALPix nside for a given angular resolution.
+
+    This function is the inverse of `nside_to_pixel_resolution`,
+    for the default rounding scheme of ``round='nearest'``.
+
+    If you choose ``round='up'``, you'll get HEALPix pixels that
+    have at least the requested resolution (usually a bit better
+    due to rounding).
+
+    Pixel resolution is defined as square root of pixel area.
+
+    Parameters
+    ----------
+    resolution : `~astropy.units.Quantity`
+        Angular resolution
+    round : {'up', 'nearest', 'down'}
+        Which way to round
+
+    Returns
+    -------
+    nside : int
+        The number of pixels on the side of one of the 12 'top-level' HEALPix tiles.
+        Always a power of 2.
+
+    Examples
+    --------
+    >>> from astropy import units as u
+    >>> from astropy_healpix import pixel_resolution_to_nside
+    >>> pixel_resolution_to_nside(13 * u.arcmin)
+    256
+    >>> pixel_resolution_to_nside(13 * u.arcmin, round='up')
+    512
+    """
+    resolution = resolution.to(u.rad).value
+    pixel_area = resolution * resolution
+    npix = 4 * math.pi / pixel_area
+    nside = np.sqrt(npix / 12)
+
+    # Now we have to round to the closest ``nside``
+    # Since ``nside`` must be a power of two,
+    # we first compute the corresponding ``level = log2(nside)`
+    # round the level and then go back to nside
+    level = np.log2(nside)
+
+    if round == 'up':
+        level = np.ceil(level)
+    elif round == 'nearest':
+        level = np.round(level)
+    elif round == 'down':
+        level = np.floor(level)
+    else:
+        raise ValueError('Invalid value for round: {!r}'.format(round))
+
+    # For very low requested resolution (i.e. large angle values), we
+    # return ``level=0``, i.e. ``nside=1``, i.e. the lowest resolution
+    # that exists with HEALPix
+    level = np.clip(level.astype(int), 0, None)
+
+    return level_to_nside(level)
+
+
 def nside_to_npix(nside):
     """
     Find the number of pixels corresponding to a HEALPix resolution.
